@@ -43,16 +43,17 @@
 
 
 
-
 #define DEFAULT_ENABLE_KEYBOARD_SHORTCUTS         TRUE
 #define DEFAULT_SHOW_NOTIFICATIONS                TRUE
-#define DEFAULT_VOLUME_STEP                       6
-#define DEFAULT_VOLUME_MAX                        153
+#define DEFAULT_VOLUME_STEP                       5
+#define DEFAULT_VOLUME_MAX                        150
 
 #ifdef HAVE_MPRIS2
 #define DEFAULT_ENABLE_MPRIS                      TRUE
+#define DEFAULT_ENABLE_MULTIMEDIA_KEYS            TRUE
 #else
 #define DEFAULT_ENABLE_MPRIS                      FALSE
+#define DEFAULT_ENABLE_MULTIMEDIA_KEYS            FALSE
 #endif
 
 #define DEFAULT_MPRIS_PLAYERS                     ""
@@ -81,6 +82,7 @@ struct _PulseaudioConfig
   GObject          __parent__;
 
   gboolean         enable_keyboard_shortcuts;
+  gboolean         enable_multimedia_keys;
   gboolean         show_notifications;
   guint            volume_step;
   guint            volume_max;
@@ -95,6 +97,7 @@ enum
   {
     PROP_0,
     PROP_ENABLE_KEYBOARD_SHORTCUTS,
+    PROP_ENABLE_MULTIMEDIA_KEYS,
     PROP_SHOW_NOTIFICATIONS,
     PROP_VOLUME_STEP,
     PROP_VOLUME_MAX,
@@ -113,6 +116,7 @@ enum
 static guint pulseaudio_config_signals [LAST_SIGNAL] = { 0, };
 
 
+
 G_DEFINE_TYPE (PulseaudioConfig, pulseaudio_config, G_TYPE_OBJECT)
 
 
@@ -136,6 +140,15 @@ pulseaudio_config_class_init (PulseaudioConfigClass *klass)
 
 
 
+  g_object_class_install_property (gobject_class,
+                                   PROP_ENABLE_MULTIMEDIA_KEYS,
+                                   g_param_spec_boolean ("enable-multimedia-keys", NULL, NULL,
+                                                         DEFAULT_ENABLE_MULTIMEDIA_KEYS,
+                                                         G_PARAM_READWRITE |
+                                                         G_PARAM_STATIC_STRINGS));
+
+
+
   g_object_class_install_property (gobject_class,
                                    PROP_SHOW_NOTIFICATIONS,
                                    g_param_spec_boolean ("show-notifications", NULL, NULL,
@@ -207,6 +220,7 @@ static void
 pulseaudio_config_init (PulseaudioConfig *config)
 {
   config->enable_keyboard_shortcuts = DEFAULT_ENABLE_KEYBOARD_SHORTCUTS;
+  config->enable_multimedia_keys    = DEFAULT_ENABLE_MULTIMEDIA_KEYS;
   config->show_notifications        = DEFAULT_SHOW_NOTIFICATIONS;
   config->volume_step               = DEFAULT_VOLUME_STEP;
   config->volume_max                = DEFAULT_VOLUME_MAX;
@@ -244,6 +258,10 @@ pulseaudio_config_get_property (GObject    *object,
       g_value_set_boolean (value, config->enable_keyboard_shortcuts);
       break;
 
+    case PROP_ENABLE_MULTIMEDIA_KEYS:
+      g_value_set_boolean (value, config->enable_multimedia_keys);
+      break;
+
     case PROP_SHOW_NOTIFICATIONS:
       g_value_set_boolean (value, config->show_notifications);
       break;
@@ -298,6 +316,16 @@ pulseaudio_config_set_property (GObject      *object,
         }
       break;
 
+    case PROP_ENABLE_MULTIMEDIA_KEYS:
+      val_bool = g_value_get_boolean (value);
+      if (config->enable_multimedia_keys != val_bool)
+        {
+          config->enable_multimedia_keys = val_bool;
+          g_object_notify (G_OBJECT (config), "enable-multimedia-keys");
+          g_signal_emit (G_OBJECT (config), pulseaudio_config_signals [CONFIGURATION_CHANGED], 0);
+        }
+      break;
+
     case PROP_SHOW_NOTIFICATIONS:
       val_bool = g_value_get_boolean (value);
       if (config->show_notifications != val_bool)
@@ -339,7 +367,14 @@ pulseaudio_config_set_property (GObject      *object,
         {
           config->enable_mpris = val_bool;
           g_object_notify (G_OBJECT (config), "enable-mpris");
-          g_signal_emit (G_OBJECT (config), pulseaudio_config_signals [CONFIGURATION_CHANGED], 0);
+
+          if (!config->enable_mpris)
+            {
+              config->enable_multimedia_keys = FALSE;
+              g_object_notify(G_OBJECT(config), "enable-multimedia-keys");
+            }
+
+          g_signal_emit(G_OBJECT(config), pulseaudio_config_signals[CONFIGURATION_CHANGED], 0);
         }
       break;
 
@@ -358,7 +393,6 @@ pulseaudio_config_set_property (GObject      *object,
 
 
 
-
 gboolean
 pulseaudio_config_get_enable_keyboard_shortcuts (PulseaudioConfig *config)
 {
@@ -369,6 +403,15 @@ pulseaudio_config_get_enable_keyboard_shortcuts (PulseaudioConfig *config)
 
 
 
+gboolean
+pulseaudio_config_get_enable_multimedia_keys (PulseaudioConfig *config)
+{
+  g_return_val_if_fail (IS_PULSEAUDIO_CONFIG (config), DEFAULT_ENABLE_MULTIMEDIA_KEYS);
+
+  return config->enable_multimedia_keys;
+}
+
+
 
 gboolean
 pulseaudio_config_get_show_notifications (PulseaudioConfig *config)
@@ -380,7 +423,6 @@ pulseaudio_config_get_show_notifications (PulseaudioConfig *config)
 
 
 
-
 guint
 pulseaudio_config_get_volume_step (PulseaudioConfig *config)
 {
@@ -391,7 +433,6 @@ pulseaudio_config_get_volume_step (PulseaudioConfig *config)
 
 
 
-
 guint
 pulseaudio_config_get_volume_max (PulseaudioConfig *config)
 {
@@ -402,7 +443,6 @@ pulseaudio_config_get_volume_max (PulseaudioConfig *config)
 
 
 
-
 const gchar *
 pulseaudio_config_get_mixer_command (PulseaudioConfig *config)
 {
@@ -413,7 +453,6 @@ pulseaudio_config_get_mixer_command (PulseaudioConfig *config)
 
 
 
-
 gboolean
 pulseaudio_config_get_enable_mpris (PulseaudioConfig *config)
 {
@@ -424,7 +463,6 @@ pulseaudio_config_get_enable_mpris (PulseaudioConfig *config)
 
 
 
-
 gchar **
 pulseaudio_config_get_mpris_players (PulseaudioConfig *config)
 {
@@ -436,12 +474,16 @@ pulseaudio_config_get_mpris_players (PulseaudioConfig *config)
   return g_strsplit (config->mpris_players, ";", 0);
 }
 
+
+
 static gint
 compare_players (gconstpointer item1, gconstpointer item2)
 {
   return g_ascii_strcasecmp (item1, item2);
 }
 
+
+
 void
 pulseaudio_config_set_mpris_players (PulseaudioConfig  *config,
                                      gchar            **players)
@@ -450,19 +492,24 @@ pulseaudio_config_set_mpris_players (PulseaudioConfig  *config,
   gchar  *player_string;
   GValue  src = { 0, };
   guint   index = 0;
+  guint   i = 0;
+  GSList *list = NULL;
 
   g_return_if_fail (IS_PULSEAUDIO_CONFIG (config));
 
   player_array = NULL;
-  for (guint i = 0; i < g_strv_length (players); i++) {
-    player_array = g_slist_prepend (player_array, players[i]);
-  }
+  for (i = 0; i < g_strv_length (players); i++)
+    {
+      player_array = g_slist_prepend (player_array, players[i]);
+    }
+
   player_array = g_slist_sort (player_array, (GCompareFunc) compare_players);
 
-  for (GSList *list = player_array; list != NULL; list = g_slist_next (list)) {
-    players[index] = list->data;
-    index++;
-  }
+  for (list = player_array; list != NULL; list = g_slist_next (list))
+    {
+      players[index] = list->data;
+      index++;
+    }
 
   g_slist_free (player_array);
 
@@ -474,6 +521,8 @@ pulseaudio_config_set_mpris_players (PulseaudioConfig  *config,
   pulseaudio_config_set_property (G_OBJECT (config), PROP_MPRIS_PLAYERS, &src, NULL);
 }
 
+
+
 void
 pulseaudio_config_add_mpris_player (PulseaudioConfig *config,
                                     gchar            *player)
@@ -484,9 +533,8 @@ pulseaudio_config_add_mpris_player (PulseaudioConfig *config,
   gchar  *player_string;
 
   players = pulseaudio_config_get_mpris_players (config);
-  if (g_strv_contains ((const char * const *) players, player)) {
-      return;
-  }
+  if (g_strv_contains ((const char * const *) players, player))
+    return;
 
   players_string = g_strjoinv (";", players);
   player_string = g_strjoin (";", players_string, player, NULL);
@@ -502,7 +550,6 @@ pulseaudio_config_add_mpris_player (PulseaudioConfig *config,
 
 
 
-
 PulseaudioConfig *
 pulseaudio_config_new (const gchar     *property_base)
 {
@@ -520,6 +567,10 @@ pulseaudio_config_new (const gchar     *property_base)
       xfconf_g_property_bind (channel, property, G_TYPE_BOOLEAN, config, "enable-keyboard-shortcuts");
       g_free (property);
 
+      property = g_strconcat (property_base, "/enable-multimedia-keys", NULL);
+      xfconf_g_property_bind (channel, property, G_TYPE_BOOLEAN, config, "enable-multimedia-keys");
+      g_free (property);
+
       property = g_strconcat (property_base, "/show-notifications", NULL);
       xfconf_g_property_bind (channel, property, G_TYPE_BOOLEAN, config, "show-notifications");
       g_free (property);
