@@ -28,6 +28,8 @@
 
 #include "pulseaudio-mpris-player.h"
 
+
+
 struct _PulseaudioMprisPlayer
 {
   GObject          __parent__;
@@ -66,10 +68,12 @@ struct _PulseaudioMprisPlayerClass
 };
 
 
+
 static void pulseaudio_mpris_player_finalize     (GObject               *object);
 static void pulseaudio_mpris_player_dbus_connect (PulseaudioMprisPlayer *player);
 
 
+
 enum
 {
   CONNECTION,
@@ -81,8 +85,11 @@ enum
 static int signals[LAST_SIGNAL] = { 0 };
 
 
+
 G_DEFINE_TYPE (PulseaudioMprisPlayer, pulseaudio_mpris_player, G_TYPE_OBJECT)
 
+
+
 static void
 pulseaudio_mpris_player_class_init (PulseaudioMprisPlayerClass *klass)
 {
@@ -119,6 +126,8 @@ pulseaudio_mpris_player_class_init (PulseaudioMprisPlayerClass *klass)
                   G_TYPE_NONE, 0);
 }
 
+
+
 static void
 pulseaudio_mpris_player_parse_metadata (PulseaudioMprisPlayer *player,
                                         GVariant              *dictionary)
@@ -133,7 +142,9 @@ pulseaudio_mpris_player_parse_metadata (PulseaudioMprisPlayer *player,
   while (g_variant_iter_loop (&iter, "{sv}", &key, &value))
     {
       if (0 == g_ascii_strcasecmp (key, "xesam:title"))
-        player->title = g_strdup (g_variant_get_string (value, NULL));
+        {
+          player->title = g_strdup(g_variant_get_string(value, NULL));
+        }
       else if (0 == g_ascii_strcasecmp (key, "xesam:artist"))
         {
           artists = g_variant_dup_strv (value, NULL);
@@ -149,10 +160,12 @@ pulseaudio_mpris_player_parse_metadata (PulseaudioMprisPlayer *player,
                 }
               g_strfreev (artists);
             }
-          }
+        }
     }
 }
 
+
+
 void
 pulseaudio_mpris_player_call_player_method (PulseaudioMprisPlayer *player,
                                             const gchar           *method)
@@ -195,6 +208,8 @@ pulseaudio_mpris_player_call_player_method (PulseaudioMprisPlayer *player,
   g_object_unref (message);
 }
 
+
+
 static GVariant *
 pulseaudio_mpris_player_get_all_player_properties (PulseaudioMprisPlayer *player)
 {
@@ -222,6 +237,8 @@ pulseaudio_mpris_player_get_all_player_properties (PulseaudioMprisPlayer *player
   return child;
 }
 
+
+
 static GVariant *
 pulseaudio_mpris_player_get_all_media_player_properties (PulseaudioMprisPlayer *player)
 {
@@ -249,6 +266,8 @@ pulseaudio_mpris_player_get_all_media_player_properties (PulseaudioMprisPlayer *
   return child;
 }
 
+
+
 static void
 pulseaudio_mpris_player_parse_playback_status (PulseaudioMprisPlayer *player,
                                                const gchar           *playback_status)
@@ -261,6 +280,8 @@ pulseaudio_mpris_player_parse_playback_status (PulseaudioMprisPlayer *player,
     player->playback_status = STOPPED;
 }
 
+
+
 static void
 pulseaudio_mpris_player_parse_player_properties (PulseaudioMprisPlayer *player,
                                                  GVariant              *properties)
@@ -272,32 +293,33 @@ pulseaudio_mpris_player_parse_player_properties (PulseaudioMprisPlayer *player,
 
   g_variant_iter_init (&iter, properties);
 
-  while (g_variant_iter_loop (&iter, "{sv}", &key, &value)) {
-    if (0 == g_ascii_strcasecmp (key, "PlaybackStatus"))
-      {
-        playback_status = g_variant_get_string(value, NULL);
-      }
-    else if (0 == g_ascii_strcasecmp (key, "CanGoNext"))
-      {
-        player->can_go_next = g_variant_get_boolean(value);
-      }
-    else if (0 == g_ascii_strcasecmp (key, "CanGoPrevious"))
-      {
-        player->can_go_previous = g_variant_get_boolean(value);
-      }
-    else if (0 == g_ascii_strcasecmp (key, "CanPlay"))
-      {
-        player->can_play = g_variant_get_boolean(value);
-      }
-    else if (0 == g_ascii_strcasecmp (key, "CanPause"))
-      {
-        player->can_pause = g_variant_get_boolean(value);
-      }
-    else if (0 == g_ascii_strcasecmp (key, "Metadata"))
-      {
-        pulseaudio_mpris_player_parse_metadata (player, value);
-        g_signal_emit (player, signals[METADATA], 0, NULL);
-      }
+  while (g_variant_iter_loop (&iter, "{sv}", &key, &value))
+    {
+      if (0 == g_ascii_strcasecmp (key, "PlaybackStatus"))
+        {
+          playback_status = g_variant_get_string(value, NULL);
+        }
+      else if (0 == g_ascii_strcasecmp (key, "CanGoNext"))
+        {
+          player->can_go_next = g_variant_get_boolean(value);
+        }
+      else if (0 == g_ascii_strcasecmp (key, "CanGoPrevious"))
+        {
+          player->can_go_previous = g_variant_get_boolean(value);
+        }
+      else if (0 == g_ascii_strcasecmp (key, "CanPlay"))
+        {
+          player->can_play = g_variant_get_boolean(value);
+        }
+      else if (0 == g_ascii_strcasecmp (key, "CanPause"))
+        {
+          player->can_pause = g_variant_get_boolean(value);
+        }
+      else if (0 == g_ascii_strcasecmp (key, "Metadata"))
+        {
+          pulseaudio_mpris_player_parse_metadata (player, value);
+          g_signal_emit (player, signals[METADATA], 0, NULL);
+        }
     }
 
   if (playback_status != NULL)
@@ -307,6 +329,8 @@ pulseaudio_mpris_player_parse_player_properties (PulseaudioMprisPlayer *player,
     }
 }
 
+
+
 static void
 pulseaudio_mpris_player_parse_media_player_properties (PulseaudioMprisPlayer *player,
                                                        GVariant              *properties)
@@ -326,6 +350,8 @@ pulseaudio_mpris_player_parse_media_player_properties (PulseaudioMprisPlayer *pl
     }
 }
 
+
+
 static void
 pulseaudio_mpris_player_on_dbus_property_signal (GDBusProxy *proxy,
                                                  gchar      *sender_name,
@@ -351,6 +377,8 @@ pulseaudio_mpris_player_on_dbus_property_signal (GDBusProxy *proxy,
   g_variant_unref (child);
 }
 
+
+
 static void
 pulseaudio_mpris_player_on_dbus_connected (GDBusConnection *connection,
                                            const gchar     *name,
@@ -376,6 +404,8 @@ pulseaudio_mpris_player_on_dbus_connected (GDBusConnection *connection,
   g_variant_unref (reply);
 }
 
+
+
 static void
 pulseaudio_mpris_player_on_dbus_lost (GDBusConnection *connection,
                                       const gchar     *name,
@@ -403,6 +433,8 @@ pulseaudio_mpris_player_on_dbus_lost (GDBusConnection *connection,
   g_signal_emit (player, signals[CONNECTION], 0, player->connected);
 }
 
+
+
 static gchar *
 find_desktop_entry (const gchar *player_name)
 {
@@ -444,6 +476,8 @@ find_desktop_entry (const gchar *player_name)
   return filename;
 }
 
+
+
 static void
 pulseaudio_mpris_player_set_details_from_desktop (PulseaudioMprisPlayer *player,
                                                   const gchar           *player_name)
@@ -489,6 +523,8 @@ pulseaudio_mpris_player_set_details_from_desktop (PulseaudioMprisPlayer *player,
   g_free (file);
 }
 
+
+
 static void
 pulseaudio_mpris_player_set_player (PulseaudioMprisPlayer *player,
                                     const gchar           *player_name)
@@ -524,6 +560,8 @@ pulseaudio_mpris_player_set_player (PulseaudioMprisPlayer *player,
   pulseaudio_mpris_player_dbus_connect (player);
 }
 
+
+
 static void
 pulseaudio_mpris_player_dbus_connect (PulseaudioMprisPlayer *player)
 {
@@ -539,7 +577,7 @@ pulseaudio_mpris_player_dbus_connect (PulseaudioMprisPlayer *player)
 
   watch_id = g_bus_watch_name_on_connection(player->dbus_connection,
                                             player->dbus_name,
-                                            G_BUS_NAME_OWNER_FLAGS_REPLACE,
+                                            G_BUS_NAME_WATCHER_FLAGS_AUTO_START,
                                             pulseaudio_mpris_player_on_dbus_connected,
                                             pulseaudio_mpris_player_on_dbus_lost,
                                             player,
@@ -591,91 +629,121 @@ pulseaudio_mpris_player_dbus_connect (PulseaudioMprisPlayer *player)
   player->watch_id = watch_id;
 }
 
+
+
 const gchar *
 pulseaudio_mpris_player_get_player (PulseaudioMprisPlayer *player)
 {
   return player->player;
 }
 
+
+
 const gchar *
 pulseaudio_mpris_player_get_player_title (PulseaudioMprisPlayer *player)
 {
   return player->player_label;
 }
 
+
+
 const gchar *
 pulseaudio_mpris_player_get_icon_name (PulseaudioMprisPlayer *player)
 {
   return player->icon_name;
 }
 
+
+
 const gchar *
 pulseaudio_mpris_player_get_title (PulseaudioMprisPlayer *player)
 {
   return player->title;
 }
 
+
+
 const gchar *
 pulseaudio_mpris_player_get_artist (PulseaudioMprisPlayer *player)
 {
   return player->artist;
 }
 
+
+
 gboolean
 pulseaudio_mpris_player_is_connected (PulseaudioMprisPlayer *player)
 {
   return player->connected;
 }
 
+
+
 gboolean
 pulseaudio_mpris_player_is_playing (PulseaudioMprisPlayer *player)
 {
   return player->playback_status == PLAYING;
 }
 
+
+
 gboolean
 pulseaudio_mpris_player_is_stopped (PulseaudioMprisPlayer *player)
 {
   return player->playback_status == STOPPED;
 }
 
+
+
 gboolean
 pulseaudio_mpris_player_can_play (PulseaudioMprisPlayer *player)
 {
   return player->can_play;
 }
 
+
+
 gboolean
 pulseaudio_mpris_player_can_pause (PulseaudioMprisPlayer *player)
 {
   return player->can_pause;
 }
 
+
+
 gboolean
 pulseaudio_mpris_player_can_go_previous (PulseaudioMprisPlayer *player)
 {
   return player->can_go_previous;
 }
 
+
+
 gboolean
 pulseaudio_mpris_player_can_go_next (PulseaudioMprisPlayer *player)
 {
   return player->can_go_next;
 }
 
+
+
 gboolean
 pulseaudio_mpris_player_can_raise (PulseaudioMprisPlayer *player)
 {
   return player->can_raise;
 }
 
+
+
 gboolean
 pulseaudio_mpris_player_is_equal (PulseaudioMprisPlayer *a,
                                   PulseaudioMprisPlayer *b)
 {
-    return g_strcmp0 (pulseaudio_mpris_player_get_player_title (a), pulseaudio_mpris_player_get_player_title (b)) == 0;
+  return g_strcmp0 (pulseaudio_mpris_player_get_player_title (a), pulseaudio_mpris_player_get_player_title (b)) == 0;
 }
 
+
+
 static void
 pulseaudio_mpris_player_init (PulseaudioMprisPlayer *player)
 {
@@ -700,6 +768,7 @@ pulseaudio_mpris_player_init (PulseaudioMprisPlayer *player)
 }
 
 
+
 static void
 pulseaudio_mpris_player_finalize (GObject *object)
 {
@@ -729,6 +798,8 @@ pulseaudio_mpris_player_finalize (GObject *object)
   (*G_OBJECT_CLASS (pulseaudio_mpris_player_parent_class)->finalize) (object);
 }
 
+
+
 PulseaudioMprisPlayer *
 pulseaudio_mpris_player_new (gchar *name)
 {
