@@ -58,6 +58,9 @@ gboolean                pulseaudio_mpris_notify_player         (PulseaudioMpris
                                                                 const gchar      *name,
                                                                 const gchar      *message);
 
+gboolean                pulseaudio_mpris_notify_any_player     (PulseaudioMpris *mpris,
+                                                                const gchar *message);
+
 G_END_DECLS
 
 #endif /* !__PULSEAUDIO_MPRIS_H__ */
