@@ -55,6 +55,7 @@ static void                 pulseaudio_volume_get_source_list_cb (pa_context
                                                                   void                 *userdata);
 
 
+
 struct _PulseaudioVolume
 {
   GObject               __parent__;
@@ -64,6 +65,8 @@ struct _PulseaudioVolume
   pa_glib_mainloop     *pa_mainloop;
   pa_context           *pa_context;
   gboolean              connected;
+  gboolean              sink_connected;
+  gboolean              source_connected;
 
   gdouble               volume;
   gboolean              muted;
@@ -91,7 +94,6 @@ struct _PulseaudioVolumeClass
 
 
 
-
 enum
 {
   VOLUME_CHANGED,
@@ -103,9 +105,10 @@ static guint pulseaudio_volume_signals[LAST_SIGNAL] = { 0, };
 
 
 
-
 G_DEFINE_TYPE (PulseaudioVolume, pulseaudio_volume, G_TYPE_OBJECT)
 
+
+
 static void
 pulseaudio_volume_class_init (PulseaudioVolumeClass *klass)
 {
@@ -179,7 +182,6 @@ pulseaudio_volume_finalize (GObject *object)
 
 
 
-
 /* sink event callbacks */
 static void
 pulseaudio_volume_sink_info_cb (pa_context         *context,
@@ -191,7 +193,10 @@ pulseaudio_volume_sink_info_cb (pa_context         *context,
   gdouble   vol;
 
   PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
-  if (i == NULL) return;
+
+  if (i == NULL)
+    return;
+
   pulseaudio_debug ("sink info: %s, %s", i->name, i->description);
 
   volume->sink_index = (guint)i->index;
@@ -203,18 +208,23 @@ pulseaudio_volume_sink_info_cb (pa_context         *context,
     {
       pulseaudio_debug ("Updated Mute: %d -> %d", volume->muted, muted);
       volume->muted = muted;
-      g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_CHANGED], 0, FALSE);
+
+      if (volume->sink_connected)
+        g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_CHANGED], 0, TRUE);
     }
 
   if (ABS (volume->volume - vol) > 2e-3)
     {
       pulseaudio_debug ("Updated Volume: %04.3f -> %04.3f", volume->volume, vol);
       volume->volume = vol;
-      g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_CHANGED], 0, FALSE);
-    }
-  pulseaudio_debug ("volume: %f, muted: %d", vol, muted);
-}
 
+      if (volume->sink_connected)
+        g_signal_emit(G_OBJECT(volume), pulseaudio_volume_signals[VOLUME_CHANGED], 0, TRUE);
+    }
+
+  pulseaudio_debug ("volume: %f, muted: %d", vol, muted);
+  volume->sink_connected = TRUE;
+}
 
 
 
@@ -228,7 +238,10 @@ pulseaudio_volume_source_info_cb (pa_context           *context,
   gdouble   vol_mic;
 
   PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
-  if (i == NULL) return;
+
+  if (i == NULL)
+    return;
+
   pulseaudio_debug ("source info: %s, %s", i->name, i->description);
 
   volume->source_index = (guint)i->index;
@@ -240,16 +253,22 @@ pulseaudio_volume_source_info_cb (pa_context           *context,
     {
       pulseaudio_debug ("Updated Mute Mic: %d -> %d", volume->muted_mic, muted_mic);
       volume->muted_mic = muted_mic;
-      g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_MIC_CHANGED], 0, FALSE);
+
+      if (volume->source_connected)
+        g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_MIC_CHANGED], 0, FALSE);
     }
 
   if (ABS (volume->volume_mic - vol_mic) > 2e-3)
     {
       pulseaudio_debug ("Updated Volume Mic: %04.3f -> %04.3f", volume->volume_mic, vol_mic);
       volume->volume_mic = vol_mic;
-      g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_MIC_CHANGED], 0, FALSE);
+
+      if (volume->source_connected)
+        g_signal_emit(G_OBJECT(volume), pulseaudio_volume_signals[VOLUME_MIC_CHANGED], 0, FALSE);
     }
+
   pulseaudio_debug ("volume mic: %f, muted mic: %d", vol_mic, muted_mic);
+  volume->source_connected = TRUE;
 }
 
 
@@ -260,7 +279,12 @@ pulseaudio_volume_server_info_cb (pa_context           *context,
                                   void                 *userdata)
 {
   PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
-  if (i == NULL) return;
+
+  if (i == NULL)
+    return;
+
+  pulseaudio_volume_set_default_input (volume, i->default_source_name);
+  pulseaudio_volume_set_default_output (volume, i->default_sink_name);
 
   pulseaudio_debug ("server: %s@%s, v.%s", i->user_name, i->server_name, i->server_version);
   pa_context_get_sink_info_by_name (context, i->default_sink_name, pulseaudio_volume_sink_info_cb, volume);
@@ -269,7 +293,6 @@ pulseaudio_volume_server_info_cb (pa_context           *context,
 
 
 
-
 static void
 pulseaudio_volume_sink_source_check (PulseaudioVolume *volume,
                                      pa_context       *context)
@@ -287,7 +310,6 @@ pulseaudio_volume_sink_source_check (PulseaudioVolume *volume,
 
 
 
-
 static void
 pulseaudio_volume_subscribe_cb (pa_context                   *context,
                                 pa_subscription_event_type_t  t,
@@ -331,9 +353,6 @@ pulseaudio_volume_subscribe_cb (pa_context                   *context,
 
 
 
-
-
-
 static void
 pulseaudio_volume_get_sink_list_cb (pa_context         *context,
                                     const pa_sink_info *i,
@@ -344,9 +363,12 @@ pulseaudio_volume_get_sink_list_cb (pa_context         *context,
   gchar *description = NULL;
 
   PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
-  if (i == NULL) return;
 
-  if (eol > 0) return;
+  if (i == NULL)
+    return;
+
+  if (eol > 0)
+    return;
 
   name = g_strdup (i->name);
   description = g_strdup (i->description);
@@ -366,12 +388,16 @@ pulseaudio_volume_get_source_list_cb (pa_context           *context,
   gchar *description = NULL;
 
   PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
-  if (i == NULL) return;
 
-  if (eol > 0) return;
+  if (i == NULL)
+    return;
+
+  if (eol > 0)
+    return;
 
   /* Ignore sink monitors, not relevant for users */
-  if (i->monitor_of_sink != PA_INVALID_INDEX) return;
+  if (i->monitor_of_sink != PA_INVALID_INDEX)
+    return;
 
   name = g_strdup (i->name);
   description = g_strdup (i->description);
@@ -387,7 +413,9 @@ pulseaudio_volume_get_server_info_cb (pa_context           *context,
                                       void                 *userdata)
 {
   PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
-  if (i == NULL) return;
+
+  if (i == NULL)
+    return;
 
   g_free (volume->default_sink_name);
   g_free (volume->default_source_name);
@@ -418,6 +446,9 @@ pulseaudio_volume_context_state_cb (pa_context *context,
       g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_CHANGED], 0, FALSE);
       g_signal_emit (G_OBJECT (volume), pulseaudio_volume_signals [VOLUME_MIC_CHANGED], 0, FALSE);
 
+      volume->sink_connected = FALSE;
+      volume->source_connected = FALSE;
+
       pa_context_get_server_info (volume->pa_context, pulseaudio_volume_get_server_info_cb, volume);
 
       break;
@@ -494,7 +525,6 @@ pulseaudio_volume_connect (PulseaudioVolume *volume)
 
 
 
-
 static gboolean
 pulseaudio_volume_reconnect_timeout  (gpointer userdata)
 {
@@ -508,8 +538,6 @@ pulseaudio_volume_reconnect_timeout  (gpointer userdata)
 
 
 
-
-
 gboolean
 pulseaudio_volume_get_connected (PulseaudioVolume *volume)
 {
@@ -557,7 +585,6 @@ pulseaudio_volume_d2v (PulseaudioVolume *volume,
 
 
 
-
 gboolean
 pulseaudio_volume_get_muted (PulseaudioVolume *volume)
 {
@@ -610,7 +637,6 @@ pulseaudio_volume_toggle_muted (PulseaudioVolume *volume)
 
 
 
-
 gboolean
 pulseaudio_volume_get_muted_mic (PulseaudioVolume *volume)
 {
@@ -682,7 +708,9 @@ pulseaudio_volume_set_volume_cb2 (pa_context         *context,
                                   void               *userdata)
 {
   PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
-  if (i == NULL) return;
+
+  if (i == NULL)
+    return;
 
   pa_cvolume_set ((pa_cvolume *)&i->volume, 1, pulseaudio_volume_d2v (volume, volume->volume));
   pa_context_set_sink_volume_by_index (context, i->index, &i->volume, pulseaudio_volume_sink_volume_changed, volume);
@@ -697,7 +725,9 @@ pulseaudio_volume_set_volume_cb1 (pa_context           *context,
                                   void                 *userdata)
 {
   PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
-  if (i == NULL) return;
+
+  if (i == NULL)
+    return;
 
   pa_context_get_sink_info_by_name (context, i->default_sink_name, pulseaudio_volume_set_volume_cb2, volume);
 }
@@ -746,7 +776,9 @@ pulseaudio_volume_set_volume_mic_cb2 (pa_context           *context,
                                       void                 *userdata)
 {
   PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
-  if (i == NULL) return;
+
+  if (i == NULL)
+    return;
 
   pa_cvolume_set ((pa_cvolume *)&i->volume, 1, pulseaudio_volume_d2v (volume, volume->volume_mic));
   pa_context_set_source_volume_by_index (context, i->index, &i->volume, pulseaudio_volume_source_volume_changed, volume);
@@ -879,7 +911,9 @@ pulseaudio_volume_default_sink_changed_info_cb (pa_context         *context,
                                                 void               *userdata)
 {
   PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
-  if (i == NULL) return;
+
+  if (i == NULL)
+    return;
 
   pa_context_move_sink_input_by_index (context, volume->sink_index, i->index, NULL, NULL);
   volume->sink_index = (guint)i->index;
@@ -902,8 +936,11 @@ pulseaudio_volume_default_sink_changed (pa_context *context,
 
 void
 pulseaudio_volume_set_default_output (PulseaudioVolume *volume,
-                                      gchar            *name)
+                                      const gchar      *name)
 {
+  if (g_strcmp0(name, volume->default_sink_name) == 0)
+    return;
+
   g_free (volume->default_sink_name);
   volume->default_sink_name = g_strdup (name);
 
@@ -919,7 +956,9 @@ pulseaudio_volume_default_source_changed_info_cb (pa_context         *context,
                                                   void               *userdata)
 {
   PulseaudioVolume *volume = PULSEAUDIO_VOLUME (userdata);
-  if (i == NULL) return;
+
+  if (i == NULL)
+    return;
 
   pa_context_move_source_output_by_index (context, volume->source_index, i->index, NULL, NULL);
   volume->source_index = (guint)i->index;
@@ -942,8 +981,11 @@ pulseaudio_volume_default_source_changed (pa_context *context,
 
 void
 pulseaudio_volume_set_default_input (PulseaudioVolume *volume,
-                                     gchar            *name)
+                                     const gchar      *name)
 {
+  if (g_strcmp0 (name, volume->default_source_name) == 0)
+    return;
+
   g_free (volume->default_source_name);
   volume->default_source_name = g_strdup (name);
 
